/**
* keygen.c - P2PWNC key/certificate generation utility
*
* Copyright (C) 2004-2005 Pantelis Frangoudis, Elias Efstathiou
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "crypt.h"

//generates user key pair and certificate
int main(int argc, char** argv) {
	if (argc < 3) {
		printf("usage: keygen <RSA/ECC> <bits> [<signer pubkey>] [<signer privkey>]\n");
		exit(1);
	}

	init_prng();

	PWNC_PUBKEY *pub = NULL;
	PWNC_PUBKEY *priv = NULL;
	PWNC_CERT *cert = NULL;
	int verified = 0;

	PWNC_KEYPAIR *pair = NULL;
	PWNC_KEYPAIR *team_pair = NULL;

	char fname[80];

	int len = 0;

	FILE *fp = NULL;

	//user key pair generation
	if (!strncasecmp(argv[1], "ecc", 3)) {
		//generate user  key pair
		pair = generate_key_pair(PWNC_ALGO_ECC, atoi(argv[2]));
		if (!pair) {
			fprintf(stderr, "Could not generate %d bit %s key pair", atoi(argv[2]), argv[1]);
			exit(1);
		}

		memset(fname, 0, 80);
		sprintf(fname, "new_ecc_pub_key_%s", argv[2]);
		write_key_file(pair->pubkey , fname);

		memset(fname, 0, 80);
		sprintf(fname, "new_ecc_priv_key_%s", argv[2]);
		write_key_file(pair->privkey , fname);
	}
	else
	if (!strncasecmp(argv[1], "rsa", 3)) {
		pair = generate_key_pair(PWNC_ALGO_RSA, atoi(argv[2]));

		memset(fname, 0, 80);
		sprintf(fname, "new_rsa_pub_key_%s", argv[2]);
		write_key_file(pair->pubkey , fname);

		memset(fname, 0, 80);
		sprintf(fname, "new_rsa_priv_key_%s", argv[2]);
		write_key_file(pair->privkey , fname);
	}
	else {
		printf("Illegal algorithm\n");
		exit(1);
	}

	//cert construction
	if (argc>=5 ){
		//read team keys

		pub = read_key_file(argv[3]);
		priv = read_key_file(argv[4]);
		ALLOC_AND_CLEAN(team_pair, sizeof(PWNC_KEYPAIR), PWNC_KEYPAIR*);

		team_pair->pubkey = pub;
		team_pair->privkey = priv;

		//CREATION of USER CERT
		cert = create_member_cert(team_pair, pair->pubkey);
		write_cert_file(cert, "newcert"); //write cert to disk

		//cert sig verification
		unsigned char *cert_info = serialize_cert_info(cert->TeamPubKey, cert->UserPubKey, &len);
		verified = verify_pwnc_signature(cert->TeamSig, pub, cert_info, len, 1);
		SAFE_FREE(cert_info);
		if (verified == 0) {
			printf("Signature not verified\n");
		}
		else
		if (verified == -1) {
			printf("Verification error\n");
		}
		else {
			printf("Verification successful\n");
		}
	}
	return 0;
}
